<?php

define('FILECOMMERCE_PATH', dirname(__FILE__) . '/');
define('FILECOMMERCE_FILE_UPLOAD_DIR', "file-commerce/");

define('FILECOMMERCE_TABLE_CONTRACTS', 'filecommerce_contracts');
define('FILECOMMERCE_TABLE_IPN', 'filecommerce_ipn');

define('FILECOMMERCE_FILE_OPTION', 'filecommerce_file');

/**
 * Update file status value ( 0 - not active, 1 - active)
 * @param $value
 */
function file_commerce_update_file_status($value) {
    global $wpdb;

    $file = get_option(FILECOMMERCE_FILE_OPTION);
    $file['status'] = $value;
    return update_option(FILECOMMERCE_FILE_OPTION, $file);
}

/**
 * Check if a user can download a file
 * @param $uid user id
 * @return
 */
function file_commerce_can_user_download_file($uid) {
	global $wpdb;

    $result = false;
    if (is_numeric($uid))
    {
    	$query = $wpdb->prepare("select count(id) as tot from " . $wpdb->prefix . FILECOMMERCE_TABLE_CONTRACTS . "
    				where status=" . FileCommerce::CONTRACT_VALID . " and expire>" . time() . " and userid = %d", array($uid));
		$res = $wpdb->get_results($query);
		if ($res[0]->tot > 0) $result = true;
    }
    return $result;
}

/**
 * Check if a file can be downloaded without buying it
 * @return bool
 */
function file_commerce_is_file_download_free() {
	global $wpdb;

    $result = false;
    $file = get_option(FILECOMMERCE_FILE_OPTION);
    if (array_key_exists('free', $file))
    	$result = $file['free'] == FileCommerce::DOWNLOAD_IS_FREE;

    return $result;
}

/**
 * Returns file name without dangerous chars
 * @param $filename
 */
function file_commerce_sanitize_filename($filename) {
	$filename = str_replace(' ', '', $filename);

	return $filename;
}

/**
 * Returns complete url of a plugin script for admin area
 * @param $script script name (without trailing slashes)
 */
function file_commerce_get_plugin_admin_url($script)  {
	return get_admin_url().'admin.php?page=file-commerce/'.$script;
}

/**
 *
 * Load a contract
 * @param $id
 */
function file_commerce_load_contract($id) {
	global $wpdb;

	if (is_numeric($id))
	{
		return $wpdb->get_row("SELECT * FROM " . $wpdb->prefix . FILECOMMERCE_TABLE_CONTRACTS . " WHERE id = " . $id);
	}
	return false;
}

/**
 *
 * Load a contract by userid
 * @param $userid
 */
function file_commerce_load_contract_by_user_and_file($userid) {
	global $wpdb;

	if (is_numeric($userid))
	{
		return $wpdb->get_row("SELECT * FROM " . $wpdb->prefix . FILECOMMERCE_TABLE_CONTRACTS . " WHERE userid = " . $userid);
	}
	return false;
}

/**
 * Delete a contract from database
 * @param $id
 */
function file_commerce_delete_contract_from_db($id) {
	global $wpdb;
    if (is_numeric($id))
    {
    	$query = $wpdb->prepare("delete from " . $wpdb->prefix . FILECOMMERCE_TABLE_CONTRACTS . " where id = " . $id);
		$r = $wpdb->query($query);
		if (!$r) return false;
    }
    else
    {
    	return false;
    }
    return true;
}

/**
 *
 * Save a contract
 * @param $userid
 * @param $fileid
 * @param $status
 * @param $expire expiration date in UNIX timestamp
 * @param $txnid transaction id, if available
 * @param $gateway payment gateway
 */
function file_commerce_save_contract($userid,$status,$expire,$txnid,$gateway) {
	global $wpdb;

	$contract = array();
    $contract['userid'] = $userid;
    $contract['status'] = $status;
    $contract['expire'] = $expire;
    $contract['txnid'] = $txnid;
    $contract['gateway'] = $gateway;

    return $wpdb->insert($wpdb->prefix . FILECOMMERCE_TABLE_CONTRACTS, $contract, array('%d','%d','%d','%s','%s'));
}

/**
 *
 * Update contract row
 * @param $id
 * @param $update_array key => value array with fields to be updated
 */
function file_commerce_update_contract($id,$update_array) {
	global $wpdb;
	if (is_numeric($id))
	{
		return $wpdb->update($wpdb->prefix . FILECOMMERCE_TABLE_CONTRACTS, $update_array, array('id'=>$id));
	}
    return false;
}

/**
 * Save ipn log
 * @param $userid
 * @param $data
 * @param $ipnval
 * @param $request
 */
function file_commerce_save_ipn($userid,$data,$ipnval='',$request='') {
	global $wpdb;

	$ipn = array();
    $ipn['userid'] = $userid;
    $ipn['data'] = $data;
    $ipn['created'] = time();
    $ipn['ipn'] = $ipnval;
    if (empty($request)) $ipn['request'] = serialize($_SERVER);
    else $ipn['request'] = $request;

    return $wpdb->insert($wpdb->prefix . FILECOMMERCE_TABLE_IPN, $ipn, array('%d','%s','%d','%s','%s'));
}

/**
 * Load ipn log
 * @param $id
 * @return ipn row as object, false in case of error
 */
function file_commerce_load_ipn($id) {
	global $wpdb;

	if (is_numeric($id))
	{
		return $wpdb->get_row("SELECT * FROM " . $wpdb->prefix . FILECOMMERCE_TABLE_IPN . " WHERE id = " . $id);
	}
	return false;
}

/**
 *
 * Update ipn row
 * @param $id
 * @param $update_array key => value array with fields to be updated
 */
function file_commerce_update_ipn($id,$update_array) {
	global $wpdb;
	if (is_numeric($id))
	{
		return $wpdb->update($wpdb->prefix . FILECOMMERCE_TABLE_IPN, $update_array, array('id'=>$id));
	}
	return false;
}
