<?php

/*
  Plugin Name: File Commerce
  Plugin URI: http://www.satollo.net/plugins/file-commerce
  Description: Sell file online quickly and simply.
  Version: 1.0.1
  Author: Satollo
  Author URI: http://www.satollo.net
  Disclaimer: Use at your own risk. No warranty expressed or implied is provided.
 */

@include_once(dirname(__FILE__) . '/commons.php');

$file_commerce = new FileCommerce();

/** Every WordPress hook managed by this class has a corresponding method prefixed
 * "hook_".
 */
class FileCommerce {

    var $title = 'File Commerce';
    var $uid; // Will be the plugin folder name and used as key for plugin options
    var $options = null;
    var $time_limit = 0;

    var $option_name_clickbank_vendor_code = 'clickbank_vendor_code';
    var $option_name_clickbank_secret_key = 'clickbank_secret_key';

    //GET params
	var $param_plugin = 'file-commerce';
    var $param_action_download = 'download';

    // Paypal related
    var $paypal_url = 'https://www.paypal.com/cgi-bin/webscr';
    //var $paypal_url = 'https://www.sandbox.paypal.com/cgi-bin/webscr';

    //Gateway value
    const GATEWAY_PAYPAL = 'paypal';

    // Contract status
    const CONTRACT_INVALID = 0; // This contract is not effective
    const CONTRACT_VALID = 1; // This status means the contract has been paid (it can be expired, thought)

    // File status
    const FILE_DISABLED = 0;
    const FILE_ON_SALE = 1;
    const FILE_ONLY_DOWNLOAD = 2;
    const FILE_FREE_REGISTERED = 3;
    const FILE_FREE = 4;
    const FILE_FREE_CONTRACT = 5; // not implemented for who have a contract active

    // File free download
    const DOWNLOAD_IS_NOT_FREE = 0;
    const DOWNLOAD_IS_FREE = 1;

    /** The constructor is called on this file load, so we keep it as light as possible. Other initializations
     * are delegated to "hook_init" called when WordPress initializes the plugins.
     */
    function __construct() {
        // Unique string id for this plugin (its folder name... it's clearly unique)
        $this->uid = basename(dirname(__FILE__));

        // Tries to compute this session time limit, but some time has been already consumed before.
        // Time limit is the 90% of PHP max execution time.
        $max_time = (int) (ini_get('max_execution_time') * 0.9);
        if ($max_time == 0) $max_time = 3600;
        $this->time_limit = time() + $max_time;

        add_action('init', array(&$this, 'hook_init'));

        // Activation and deactivation hooks (cannot be placed on "hook_init", that
        // function is not called on activation)
        register_activation_hook(__FILE__, array(&$this, 'hook_activate'));
        register_deactivation_hook(__FILE__, array(&$this, 'hook_deactivate'));
    }

    function hook_init() {
        // Checks if there is a PayPal IPN call
        //$this->paypal_ipn();

    	//including download script
    	if ($_GET['plugin'] == $this->param_plugin) {
            $action = $_GET['action'];
            if ($action == $this->param_action_download)
                    include(dirname(__FILE__) . '/download.php');
        }
        add_action('admin_menu', array(&$this, 'hook_admin_menu'));
        add_action('admin_head', array(&$this, 'hook_admin_head'));
        add_action('register_form', array(&$this, 'hook_register_form'));
        add_action('wp_head', array(&$this, 'hook_wp_head'));

        add_filter('login_message', array(&$this, 'hook_login_message'));
        add_shortcode('file_commerce', array(&$this, 'shortcode_file_commerce'));
        add_shortcode('file_commerce_thankyou', array(&$this, 'shortcode_file_commerce_thankyou'));
    }

    function hook_login_message($message) {
        return $message . $this->get_option('login_message');
    }

    /**
     * Adds a message on botton of WordPress registration form.
     */
    function hook_register_form() {
        echo '<p>';
        echo $this->get_option('register_message');
        echo '</p>';
    }

    function shortcode_file_commerce($attrs, $content) {
        global $wpdb, $current_user;

        $action = $_GET['action'];
        if (!isset($action)) $action = 'listing';

        get_currentuserinfo();

        if ($action == 'listing')
        {
	        $buffer = $this->do_list_available_file();
        }

        return $buffer;
    }

    function shortcode_file_commerce_thankyou($attrs, $content) {
        if (!isset($_GET['thankyou'])) return '';

        if (is_user_logged_in()) {
            $content .= $this->do_list_available_file();
        }

        return $content;
    }

    function do_list_available_file() {
    	global $wpdb, $current_user;

        $file = get_option(FILECOMMERCE_FILE_OPTION);
        if (!is_array($file)) return;

        $thankyou = isset($_GET['thankyou']);

        $contracts = $wpdb->get_results("select * from " . $wpdb->prefix . FILECOMMERCE_TABLE_CONTRACTS .
            " where userid = " . $current_user->ID . " and status = " .
            FileCommerce::CONTRACT_VALID . " and expire > " . time() . " order by expire desc");

        if ($thankyou && empty($contracts)) return;

        $buffer .= '<div class="file-commerce-file">';
        $buffer .= '<div class="file-commerce-title">' . $file['title'] . '</div>';
        if (!empty($file['description'])) {
            $buffer .= '<div class="file-commerce-description">' . $file['description'] . '</div>';
        }
        if (!$thankyou && $file['status'] == FileCommerce::FILE_ON_SALE) {
            $buffer .= '<div class="file-commerce-price">' . $file['price'] . '</div>';
        }

            // Used on download.php too, to be moved in a function!
            $verified = false;
            switch ($file['status']) {
                case FileCommerce::FILE_ON_SALE:
                case FileCommerce::FILE_ONLY_DOWNLOAD:
                    $verified = !empty($contracts);
                    break;

                case FileCommerce::FILE_FREE_REGISTERED:
                    $verified = is_user_logged_in();
                    break;
            }

            if ($verified) {
                $buffer .= $this->download_button($file);
            }

            if (!$thankyou) {
                if (!is_user_logged_in()) {
                    $buffer .= '<div class="file-commerce-login">';
                    $buffer .= '<a href="' . wp_login_url( get_permalink() ) . '">' . $this->get_option('login_label') . '</a>';
                    $buffer .= '</div>';
                }
                else {
                    if (!$verified && $file['status'] == FileCommerce::FILE_ON_SALE) {
                        $buffer .= '<div class="file-commerce-buttons">';
                        if ($file['paypal'] != '') $buffer .= $this->paypal_button($file);
                        $buffer .= '</div>';
                    }
                }

            }

            $buffer .= '</div>';

            return $buffer;
            
    }

    function download_button() {
        return '<div class="file-commerce-download"><a href="' . get_bloginfo('wpurl'). "/?plugin=".$this->param_plugin."&action=".$this->param_action_download . '">' . $this->get_option('download_label') . '</a></div>';
    }

    function paypal_button($file) {
        global $current_user;
        $buffer .= '<a href="' . $this->paypal_button_url($file, $current_user->ID) . '">';
        $buffer .= '<img src="' . plugins_url('img/paypal.jpg', __FILE__) . '"/>';
        $buffer .= '</a> ';
        return $buffer;
    }

    function paypal_button_url($file, $user_id) {
        $return = $this->get_option('main_url');
        if (strpos($_SERVER['REQUEST_URI'], '?') === false) $return .= '?thankyou';
        else $return .= '&thankyou';
        return $this->paypal_url . '?cmd=_s-xclick&hosted_button_id=' . $file['paypal'] .
                '&amp;notify_url=' . urlencode(plugins_url('paypal.php', __FILE__)) .
                '&amp;custom=' . $user_id .
                '&amp;return=' . urlencode($return);
    }
    
    function hook_admin_menu() {
        add_menu_page($this->title, $this->title, 10, $this->uid . '/user_guide.php', '', '');
        add_submenu_page($this->uid . '/user_guide.php', 'User Guide', 'User Guide', 10, $this->uid . '/user_guide.php');
        add_submenu_page($this->uid . '/user_guide.php', 'Configuration', 'Configuration', 10, $this->uid . '/main.php');
        add_submenu_page($this->uid . '/user_guide.php', 'File', 'File', 10, $this->uid . '/repository_edit.php');
        add_submenu_page($this->uid . '/user_guide.php', 'Contracts', 'Contracts', 10, $this->uid . '/contracts.php');
        add_submenu_page($this->uid . '/contracts.php', 'New contract', 'New contract', 10, $this->uid . '/contracts_add.php');
        add_submenu_page($this->uid . '/contracts.php', 'Edit contract', 'Edit contract', 10, $this->uid . '/contracts_edit.php');
    }

    function hook_admin_head() {
    	if ( strpos($_SERVER['QUERY_STRING'] , 'file-commerce') !== false )
    	{
        	echo '<script type="text/javascript" src="'.plugins_url('/js/admin.js', __FILE__).'"></script>';
        	echo '<link rel="stylesheet" href="'.plugins_url('/css/admin.css', __FILE__).'" type="text/css" media="all" />';
    	}
    }

    function log($text, $level) {
        if ($this->get_option('log') < $level) return;
        $time = date('d-m-Y H:i:s ');
        switch ($level) {
            case 1: $time .= '- ERROR - ';
                break;
            case 2: $time .= '- INFO  - ';
                break;
            case 3: $time .= '- DEBUG - ';
                break;
        }
        if (is_array($text) || is_object($text)) $text = print_r($text, true);
        file_put_contents(dirname(__FILE__) . '/log.txt', $time . ' - ' . $text . "\n", FILE_APPEND | FILE_TEXT);
    }

    function log_error($text) {
        $this->log($text, 1);
    }

    function log_info($text) {
        $this->log($text, 2);
    }

    function log_debug($text) {
        $this->log($text, 3);
    }

    /** Returns plugin default options. Modify here adding values or loading from file when language dependant. */
    function get_default_options() {
        $sitename = strtolower($_SERVER['SERVER_NAME']);
        if (substr($sitename, 0, 4) == 'www.') $sitename = substr($sitename, 4);
        $thank_you = 'Thank you for your purchase. If the file cannot be downloaded right now, be patience, it can take some for ayment gateway to notify it (if you paid with eCheck it can take 2 or 3 days).';

        $notify_ok_subject = "New contract active!";
        $notify_ok_body = "Thank you for your purchase. The file:\n\n{file_title}\n\ncan now be downloaded and it will be granted until:\n\n" .
            "{contract_expire}\n\nYour user name to log in the site and download the file is '{user_login}'. For any question report" .
            "the contract number '{contract_id}'.\n\n Have a nice day!";

        $notify_ko_subject = "Contract invalidated";
        $notify_ko_body = "Hi,\n\nThe contract '{contract_id}' for the file\n\n{file_title}\n\nhas been invalidated.\n\n" .
            "For any question, write us.\n\nHave a nice day.";

        $support_ok = '<p>Your request has been sent. Thank you!</p>';
        $support_login = '<p>You must log in to send support requests. <a href="{login_url_back}">Log in here</a>.</p>';

        return array('log' => 0, 'sender_name' => get_option('blogname'), 'sender_email' => 'wordpress@' . $sitename,
            'thank_you'=>$thank_you, 
            'notify_ok_subject'=>$notify_ok_subject,
            'notify_ok_body'=>$notify_ok_body,
            'notify_ko_subject'=>$notify_ko_subject,
            'notify_ko_body'=>$notify_ko_body,
            'login_label'=>'Log in or register',
            'download_label'=>'DOWNLOAD NOW!',
            'log'=>0
            );
    }

    /** Returns the plugin options. */
    function get_options($suffix = '') {
        // Non main options are loaded each time
        if ($suffix != '') return get_option($this->uid . $suffix, array());
        if ($this->options == null) $this->options = get_option($this->uid, array());
        return $this->options;
    }

    function set_options($options, $suffix = '') {
        update_option($this->uid . $suffix, $options);
        if ($suffix == '') $this->options = $options;
    }

    /** Returns a plugin option by name, only for main options */
    function get_option($name, $default=null) {
        if ($this->options == null) $this->options = get_option($this->uid, array());
        $value = $this->options[$name];
        return isset($value) ? $value : $default;
    }

    function hook_activate() {
        global $wpdb;

        // OPTIONS //
        update_option($this->uid, array_merge($this->get_default_options(), get_option($this->uid, array())));
        add_option($this->uid . '_contracts', array(), null, 'no');

        // DATABASE //
        
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        $table_name = $wpdb->prefix . FILECOMMERCE_TABLE_CONTRACTS;
        $sql = "CREATE TABLE " . $table_name . " (
            id int(11) NOT NULL AUTO_INCREMENT,
            userid int(11) NOT NULL DEFAULT 0,            
            status tinyint(4) NOT NULL DEFAULT 0,
            expire bigint(20) DEFAULT 0 NOT NULL,
            txnid varchar(50) not null default '',
            gateway varchar(20) not null default '',
            PRIMARY KEY id (id)
            );";
        dbDelta($sql);

        $table_name = $wpdb->prefix . FILECOMMERCE_TABLE_IPN;
        $sql = "CREATE TABLE " . $table_name . " (
            id mediumint(9) NOT NULL AUTO_INCREMENT,
            userid mediumint(9) NOT NULL,            
            data text,
            created bigint(20),
            ipn varchar(20),
            request text,
            PRIMARY KEY id (id)
            );";
        dbDelta($sql);
        
        // CUSTOM UPLOAD DIR //
        
        //create upload custom dir into wp upload dir
        $custom_upload_dir = $this->get_wp_upload_dir_path() . DIRECTORY_SEPARATOR . FILECOMMERCE_FILE_UPLOAD_DIR;
        if ( ! file_exists( $custom_upload_dir ) )
        {
        	if ( ! mkdir( $custom_upload_dir ) )
            {
            	
            }
        }
		//copy some useful files into custom upload dir to prevent direct access to folder
        @copy( dirname(__FILE__).'/files/index.php' , $custom_upload_dir . 'index.php' );
        @copy( dirname(__FILE__).'/files/.htaccess' , $custom_upload_dir . '.htaccess' );
        @copy( dirname(__FILE__).'/files/web.config' , $custom_upload_dir . 'web.config' );
    }

    function hook_deactivate() {

    }

    /************************************
      				PAYMENTS
     ************************************/

    /**
     * Returns contracts expiration date in UNIX timestamp
     * @param $file_duration duration value for the file
     */
    function get_contract_expiration_value($file_duration) {
    	return time() + $file_duration * 24 * 3600;
    }

    /**
     * Generate an "From" header, if needed, to get the name and email from main configuration. If an email
     * is not specified an empty tring is returned so the mail will be sent with the default WordPress
     * "From" header.
     */
    function mail_from_header() {
        $sender_email = $this->get_option('sender_email');
        if (!empty($sender_email)) {
            return 'From: ' . $this->get_option('sender_name') . '<' . $sender_email . '>' . "\n";
        }
        return '';
    }

    /**
     * Send out an email, logging it.
     */
    function mail($to, $subject, $body, $headers='') {
        $this->log_debug('Mailing to ' . $to);
        $this->log_debug('Subject: ' . $subject);

        $r = wp_mail($to, $subject, $body, $headers);
        if (!$r) $this->log_info('Mail failed');
        return $r;
    }

    /**
     *
     * Notify a new contract creation by email
     * @param $file obj
     * @param $user obj
     */
    function notify_new_contract($contract) {

        $body = $this->get_option('notify_ok_body');
        $body = $this->replace($body, $contract);
        $user = get_user_by('id', is_object($contract)?$contract->userid:$contract['userid']);
        $this->mail($user->user_email, $this->get_option('notify_ok_subject'), $body);

		$message = 'A new file has just been sold:' . "\n\n";
        $message .= 'file: ' . $file->title . "\n";
        $message .= 'user login: ' . $user->user_login . "\n";
        $message .= 'user email: ' . $user->user_email . "\n";
        $message .= 'user id: ' . $user->ID . "\n";
        $message .= "\n";
        $message .= "Excellent, yours File Commerce";

        $this->mail(get_option('admin_email'), '[' . get_option('blogname') . '] New file sold', $message);
    }

    /**
     *
     * Notify a contract invalidation by email
     * @param $contract obj
     * @param $user obj
     * @param $reason
     */
    function notify_invalidated_contract($contract) {
        // Notify the user about the new contract status
        $body = $this->get_option('notify_ko_body');
        $body = $this->replace($body, $contract);
        $user = get_user_by('id', is_object($contract)?$contract->userid:$contract['userid']);
        $this->mail($user->user_email, $this->get_option('notify_ko_subject'), $body);

        // Notify the admin
        $message = "A contract has been invalidated:\n\n";
        $message .= "file: {file_title}\n";
        $message .= "user login: {user_login}\n";
        $message .= "user email: {user_email}\n";
        $message .= "user id: {user_id}\n";
        $message .= "gateway: {contract_gateway}\n";
        $message .= 'reason: ' . $reason . "\n";

        $message .= "\n";
        $message .= "Don't worry for one lost two gained. Yours, File Commerce";

        $message = $this->replace($message, $contract);

        $this->mail(get_option('admin_email'), '[' . get_option('blogname') . '] Contract invalidated', $message);
    }

    /** This is a generic replacement function that can be used anywhere. To replace only user data, pass only the
     * $user. If a contract (both object or array) is passed and the user and file are null, they will be extracted
     * automatically.
     */
    function replace($text, $contract = null, $user = null, $file = null)
    {
        if ($contract != null) {
          if ($user == null) $user = get_user_by('id', is_object($contract)?$contract->userid:$contract['userid']);
          if ($file == null) //$file = file_commerce_load_file_from_db(is_object($contract)?$contract->fileid:$contract['fileid']);
          	$file = get_option(FILECOMMERCE_FILE_OPTION);
        }

        if ($file != null) {
            $text = str_replace('{file_title}', strip_tags($file['title']), $text);
            //$text = str_replace('{file_id}', strip_tags($file->id), $text);
        }

        if ($contract != null) {
            $text = str_replace('{contract_expire}', date('Y/m/d', is_object($contract)?$contract->expire:$contract['expire']), $text);
            $text = str_replace('{contract_gateway}', is_object($contract)?$contract->gateway:$contract['gateway'], $text);
            $text = str_replace('{contract_id}', is_object($contract)?$contract->id:$contract['id'], $text);
        }

        if ($user != null) {
            $text = str_replace('{user_id}', $user->ID, $text);
            $text = str_replace('{user_login}', $user->user_login, $text);
            $text = str_replace('{user_email}', $user->user_email, $text);
        }
        // Generic login, right for emails too
        $text = str_replace('{login_url}', wp_login_url(get_option('home')), $text);
        // Login that redirects to the current page
        $text = str_replace('{login_url_back}', wp_login_url(get_permalink()), $text);

        return $text;
    }

    /**
     * Returns a list of ACTIVE contract for the user and the file passed. Active contracts
     * are the ones with VALID status and NOT EXPIRED.
     */
    function get_active_contracts($user_id) {
        global $wpdb;

        return $wpdb->get_results("select * from " . $wpdb->prefix . FILECOMMERCE_TABLE_CONTRACTS .
            " where userid = " . $user_id . " and status = " .
            FileCommerce::CONTRACT_VALID . " and expire > " . time() . " order by expire desc");
    }

    function hook_wp_head() {
        echo '<link rel="stylesheet" type="text/css" href="' . plugins_url('css/style.css', __FILE__) . '"/>';
        echo $this->get_option('style');
    }
    
    /**
     * Returns wp upload dir base path, without trailing slash     
     */
    function get_wp_upload_dir_path() {
    	$uploads = wp_upload_dir();
    	return $uploads['basedir'];
    }
    
    /**
     * Returns wp upload dir base url, without trailing slash     
     */
    function get_wp_upload_dir_url() {
    	$uploads = wp_upload_dir();
    	return $uploads['baseurl'];
    }

}
